package customnode;

import javax.vecmath.Vector3f;
import javax.vecmath.Point3f;

import java.util.Map;
import java.util.HashMap;
import java.util.List;

public class CheckMesh {

	private List<Point3f> mesh;
	private int edgesWithLessThanTwoTriangles;
	private int edgesWithMoreThanTwoTriangles;
	private int edgesWithTwoTriangles;
	private int trianglesWithZeroArea;

	public CheckMesh(List<Point3f> mesh) {
		this.mesh = mesh;
	}

	public void check() {
		Map<Edge, Edge> edges = new HashMap<Edge, Edge>();
		Vector3f v1 = new Vector3f();
		Vector3f v2 = new Vector3f();
		trianglesWithZeroArea = 0;
		for(int i = 0; i < mesh.size(); i += 3) {
			System.out.print(i + " / " + mesh.size() + "    \r");
			Point3f p1 = mesh.get(i);
			Point3f p2 = mesh.get(i + 1);
			Point3f p3 = mesh.get(i + 2);
			// area
			v1.sub(p2, p1);
			v2.sub(p3, p1);
			v1.cross(v1, v2);
			float a = v1.length();
			if(a < 0.0001f)
				trianglesWithZeroArea++;
			// manifoldness
			Edge e = new Edge(p1, p2);
			if(edges.containsKey(e))
				e = edges.get(e);
			else
				edges.put(e, e);
			e.addTriangle();

			e = new Edge(p2, p3);
			if(edges.containsKey(e))
				e = edges.get(e);
			else
				edges.put(e, e);
			e.addTriangle();

			e = new Edge(p3, p1);
			if(edges.containsKey(e))
				e = edges.get(e);
			else
				edges.put(e, e);
			e.addTriangle();
		}

		edgesWithLessThanTwoTriangles = 0;
		edgesWithMoreThanTwoTriangles = 0;
		edgesWithTwoTriangles = 0;

		for(Edge e : edges.keySet()) {
			int n = e.nTriangles;
			if(n < 2) edgesWithLessThanTwoTriangles++;
			else if(n > 2) edgesWithMoreThanTwoTriangles++;
			else edgesWithTwoTriangles++;
		}
		System.out.println(edgesWithLessThanTwoTriangles +
			" edges with less than 2 triangles");
		System.out.println(edgesWithMoreThanTwoTriangles +
			" edges with more than 2 triangles");
		System.out.println(edgesWithTwoTriangles +
			" edges with 2 triangles");
		System.out.println(trianglesWithZeroArea +
			" trianglesWithZeroArea");
	}

	public int getEdgesWithLessThanTwoTriangles() {
		return edgesWithLessThanTwoTriangles;
	}

	public int getEdgesWithMoreThanTwoTriangles() {
		return edgesWithMoreThanTwoTriangles;
	}

	public int getEdgesWithTwoTriangles() {
		return edgesWithTwoTriangles;
	}

	public boolean inClosed() {
		return edgesWithLessThanTwoTriangles == 0;
	}

	public boolean isManifold() {
		return edgesWithMoreThanTwoTriangles == 0;
	}

	private class Edge {
		private final Point3f start, end;
		private int nTriangles;

		public Edge(Point3f start, Point3f end) {
			this.start = start;
			this.end = end;
			this.nTriangles = 0;
		}

		public void addTriangle() {
			nTriangles++;
		}

		public boolean equals(Object o) {
			if (!(o instanceof Edge))
				return false;

			Edge e = (Edge) o;
			return (start.equals(e.end) && end.equals(e.start))
				|| (end.equals(e.end) && start.equals(e.start));
		}

		public int hashCode() {
			return start.hashCode() * end.hashCode();
		}
	}
}

